/*
   ; K-TEAM debug calls library
   ; ==========================

   ;------------------------------------------------------------------------
   ; Author:    Jean Marc Koller 06/98
   ; Modifs:    Olivier Carmona  07/99
   ;
   ; Project:   KHEPERA
   ; Goal:              Include file needed for the K-TEAM Debug Library
   ;
   ;
   ;------------------------------------------------------------------------
 */

#ifndef _KTDEBUG_H
#define _KTDEBUG_H

#ifdef  __cplusplus
extern "C"
{
#endif

/*****************************************************************************/
/*                           JMK Bios New Defines                            */

#define mot_config_pid mot_config_speed_1m
#define mot_new_speed_pid mot_new_speed_1m
#define mot_put_position mot_put_sensors_1m

#define KBIOS_KHEPERA           1	/* Khepera minirobot */
#define KBIOS_KOALA             0	/* Koala minirobot */

/*  control module */

#define KMOTPIDSample           0	/* sync with PID sample interruption       */
#define KMOTOnTarget            1	/* sync with PID (position mode) on target */
#define KMSGMessageSent         2	/* sync with message sent                  */
#define KMSGMessageReceived     3	/* sync with message revceived             */
#define KSERMessageSent         4	/* sync with message sent                  */
#define KSERByteReceived        5	/* sync with byte revceived                */
#define KSENSStartSweep         6	/* sync with start IR sequence             */
#define KVARInt                 7	/* sync with module VAR interruption       */

/* intercommunication module */

#define KMSG_CH_RDY             0	/* msg channel ready                       */
#define KMSG_CH_BUSY           -1	/* msg channel busy                        */
#define KMSG_OK                 0	/* msg sent or received correctly          */
#define KMSG_TO_ERR            -1	/* msg timeout error                       */

/* various gadgets control module */

#define KVAR_CPU_16MHz          0	/* CPU at 16MHz (normal operations)        */
#define KVAR_CPU_8MHz           1	/* CPU at 8MHz (slow down operations)      */
#define KVAR_LED0               0	/* led number 0                            */
#define KVAR_LED1               1	/* led number 1                            */

/* serial line control module */

#define KSER_CH_RDY             0	/* ser channel ready                       */
#define KSER_CH_BUSY           -1	/* ser channel busy                        */
#define KSER_BD_DEFAULT         0	/* ser baud-rate default value 9600 bits/s */
#define KSER_BD_600             1	/* ser baud-rate value 600 bits/s          */
#define KSER_BD_1200            2	/* ser baud-rate value 1200 bits/s         */
#define KSER_BD_4800            3	/* ser baud-rate value 4800 bits/s         */
#define KSER_BD_9600            4	/* ser baud-rate value 9600 bits/s         */
#define KSER_BD_19200           5	/* ser baud-rate value 19200 bits/s        */
#define KSER_BD_38400           6	/* ser baud-rate value 38400 bits/s        */
#define KSER_BD_57600           7	/* ser baud-rate value 57600 bits/s        */
#define KSER_SND_OK             0	/* ser buffer sent correctly               */
#define KSER_REC_BUF_EMPTY     -1	/* ser reception buffer empty              */
#define KSER_SND_BUF_FULL      -1	/* ser sent buffer full                    */
#define KSER_SZ_TO_BIG         -2	/* ser size of the buffer too big          */
#define KSER_SZ_NULL           -3	/* ser size of the buffer = 0              */

/* protocol string conversion module */

#define KSTR_CNV_OK             0	/* str conversion OK                       */
#define KSTR_FMT_ERR           -1	/* str format error                        */
#define KSTR_SKIP_OK            0	/* str skipping OK                         */
#define KSTR_TOO_PARAM         -1	/* str too many parameters                 */

/*****************************************************************************/
/*                  Prototype of KT Debug Functions                          */


/* Initialisation of the library (always needed) */

  void KTdebug_init (void);

/* Initialisation for each kind of variable */

  unsigned char KTdebug_addWatch_Boolean (char *name);

  enum IntegerModes
    {
      NONE = 0,
      MODE_SLIDER = (1 << 0),
      MODE_DECIMAL = (1 << 1),
      MODE_HEXA = (1 << 2),
      MODE_BINARY = (1 << 3),
      MODE_MINMAX = (1 << 4)
    };

  unsigned char KTdebug_addWatch_Integer (char *name, enum IntegerModes mode, unsigned char size, unsigned char isUnsigned);

  unsigned char KTdebug_addWatch_IntegerMMO (char *name, enum IntegerModes mode, unsigned char size, unsigned char isUnsigned, int min, int max, int offset);

  struct VarArrayDesc
    {
      char size;
      char alpha;
    };

  unsigned char KTdebug_addWatch_Array (char *name, enum IntegerModes mode, unsigned char nbvars, struct VarArrayDesc varray[]);

  unsigned char KTdebug_addWatch_ArraySingleType (char *name, enum IntegerModes mode, unsigned char nbvars, char size, char *alpha, int min, int max);

/* Force update refresh of variable */

  void KTdebug_update (unsigned char handler, void *pvalue);

#define KTdebug_updateBoolean(h,v) KTdebug_update(h,(void*)&v)
#define KTdebug_update8(h,v) KTdebug_update(h,(void*)&v)
#define KTdebug_update16(h,v) KTdebug_update(h,(void*)&v)
#define KTdebug_update32(h,v) KTdebug_update(h,(void*)&v)
#define KTdebug_updateArray(h,v) KTdebug_update(h,(void*)v)

/* Add automatic refresh of variable */

  void KTdebug_setAutoWatch (unsigned char handler, void *ptr);

  void KTdebug_disAutoWatch (unsigned char handler);

/* Add an action button */

  typedef void (*EventCallBackFunc) (int);

  unsigned char KTdebug_addActor_Button (char *name, EventCallBackFunc pCBfunc);

  unsigned char KTdebug_addActor_Value (char *name, enum IntegerModes mode, int initial, int min, int max, EventCallBackFunc pCBfunc);

/* Freeze the robot */

  void KTdebug_Freeze (void);


#ifndef KT_INCLUDE
#define	ser_receive_byte	KT_ser_receive_byte
#define	ser_rx_status	KT_ser_rx_status
#endif	/* KT_INCLUDE */

  void startSpySensors (enum IntegerModes mode);

#ifdef  __cplusplus
}
#endif

#endif	/* _KTDEBUG_H */
