/*
 * Terminal.java
 * June 1998, Jean-Marc Koller
 */

/* Main programm of kheperaTools */

package KheperaTools;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JToggleButton;
import javax.swing.JTextArea;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JLabel;
import javax.swing.UIManager;

public class Terminal implements KeyListener, 
WindowListener, SerialPortEventListener, ActionListener
{
  SerialPort serialPort;
  OutputStream serialOutput;
  InputStream serialInput;
  JFrame terminalFrame;
  JTextArea terminalText;
  JScrollPane scrollpane;
  JPanel but_panel;
  JToggleButton but_freeze, but_analyze;

  InstrumentPanel instruments;

  void createMenu (JFrame frame)
  {
    JMenuBar mbar = new JMenuBar ();
    JMenu menu = new JMenu ("File");
    JMenuItem item;
      menu.add (item = new JMenuItem ("Upload file"));
      item.addActionListener (this);
      menu.addSeparator ();
      menu.add (item = new JMenuItem ("Exit"));
      item.addActionListener (this);
      mbar.add (menu);
      menu = new JMenu ("?");
      menu.add (item = new JMenuItem ("About..."));
      item.addActionListener (this);
      mbar.add (menu);
      frame.setJMenuBar (mbar);
  }

  void openTerminalWindow ()
  {
    terminalFrame = new JFrame ("Terminal");
    terminalText = new JTextArea (25, 80);
    terminalText.addKeyListener (this);
    scrollpane = new JScrollPane ();
    scrollpane.getViewport ().add (terminalText);
    scrollpane.setRowHeaderView ();
    createMenu (terminalFrame);
    but_panel = new JPanel ();
    but_freeze = new JToggleButton ("FREEZE");
    but_freeze.addActionListener (this);
    but_panel.add (but_freeze);
    but_analyze = new JToggleButton ("ANALYZE");
    but_analyze.addActionListener (this);
    but_panel.add (but_analyze);
    terminalFrame.getContentPane ().add ("North", but_panel);
    terminalFrame.getContentPane ().add ("Center", scrollpane);
    terminalFrame.addWindowListener (this);
    terminalFrame.pack ();
    terminalFrame.show ();
  }

  public void keyTyped (KeyEvent e)
  {
    try
    {
      serialOutput.write (e.getKeyChar ());
    }
    catch (IOException ioe)
    {
      System.out.println (ioe);
    }
  }
  public void keyReleased (KeyEvent e) { }
  public void keyPressed (KeyEvent e) { }

  public void windowClosed (WindowEvent e) { }
  public void windowDeiconified (WindowEvent e) { }
  public void windowIconified (WindowEvent e) { }
  public void windowActivated (WindowEvent e) { }
  public void windowDeactivated (WindowEvent e) { }
  public void windowOpened (WindowEvent e) { }
  public void windowClosing (WindowEvent e)
  {
    System.exit (0);
  }

  public void actionPerformed (ActionEvent e)
  {
    String action = e.getActionCommand ();
    if (action.equals ("Exit"))
        System.exit (0);
    if (action.equals ("Upload file"))
      {
	FileDialog uf = new FileDialog (terminalFrame, "Upload S to Khepera");
	  uf.setMode (FileDialog.LOAD);
	  uf.show ();
	if (uf.getFile () != null)
	  {
	    try
	    {
	      new upLoad (uf.getDirectory () + uf.getFile (), serialOutput);
	    }
	    catch (IOException ioe)
	    {
	    }
	  }
	return;
      }
    if (action.equals ("About..."))
      {
       JOptionPane pane = new JOptionPane(new JLabel( "About KheperaTools..."));
       JDialog dialog = pane.createDialog(terminalFrame, "About KheperaTools..." );
       dialog.show();
       return;
      }
    if (action.equals ("FREEZE"))
      {
	String toFreeze = "@IfReEzeK";
	String toThaw = "@IthawK";
	sendData ((!but_freeze.isSelected ())? toThaw.getBytes () : toFreeze.getBytes ());
	sendData ((!but_freeze.isSelected ())? toThaw.getBytes () : toFreeze.getBytes ());
	return;
      }
    if (action.equals ("ANALYZE"))
      {
	AnalyzePanel ap = new AnalyzePanel (instruments.getInstrumentArray (), serialPort);
      }
    System.out.println ("action=" + e);
  }

  public void serialEvent (SerialPortEvent ev)
  {
    switch (ev.getEventType ())
      {
      case ev.DATA_AVAILABLE:
	try
	{
	  byte[]buffer = new byte[128];
	  do
	    {
	      int len = serialInput.read (buffer);
	      if (instruments != null)
		  instruments.HandleData (buffer, len);
	        displayCleanBuffer (new String (buffer, 0, len));
	    }
	  while (serialInput.available () > 0);
	}
	catch (IOException ioe)
	{
	  System.out.println (ioe);
	}
	break;
      }
  }

  private boolean persistantHide = false;

  public void displayCleanBuffer (String in)
  {
    if (terminalText == null)
      return;
    int begin = 0;
    int sindex = (persistantHide) ? 0 : in.indexOf ("@I");
    if (sindex != -1)
      {
	int eindex = in.indexOf ('K');
	if (eindex != -1)
	  {
	    StringBuffer ws = new StringBuffer (in.length ());
	    while (sindex != -1 && eindex != -1)
	      {
		if (persistantHide)
		  persistantHide = false;
		else
		  {
		    if (sindex > begin)
		      ws.append (in.substring (begin, sindex));
		  }
		begin = eindex + 1;
		sindex = in.indexOf ("@I", begin);
		eindex = (sindex == -1) ? -1 : in.indexOf ('K', sindex);
	      }
	    persistantHide = (sindex != -1 && eindex == -1);
	    if (sindex == -1 && !persistantHide)
	      ws.append (in.substring (begin));
	    terminalText.append (ws.toString ());
	    /* scrollpane.move */
	    return;
	  }
	else
	  {
	    persistantHide = true;
	    return;
	  }
      }
    persistantHide = false;
    terminalText.append (in);
    /* scrollpane.move */
  }

  public void sendData (byte[]data)
  {
    try
    {
      serialOutput.write (data);
    }
    catch (IOException ioe)
    {
      System.out.println ("Sending Data: "+ioe);
    }
  }

  private static void usage ()
  {
    System.out.println ("KheperaTools.Terminal: 1998 Jean-Marc Koller");
    System.out.println ("       -help           : this message");
    System.out.println ("usage: -port <comport> : specify a comport (default:\"COM1\")");
    System.out.println ("       -upload <file>  : uploads a Srecord file to Khepera");
    System.out.println ("       -speed <speed>  : specify a baudrate for the port (default: 38400)");
  }

  static SerialPort searchSerialPort (String port) throws NoSuchPortException, PortInUseException
  {
    Enumeration portList;
    CommPortIdentifier portId = null;
    
    portId = CommPortIdentifier.getPortIdentifier (port);
    System.out.println ("using " + portId.getName ());
    return (SerialPort) portId.open ("KheperaLoad", 400);
  }

  public Terminal (String port, int speed)
  {
    try
    {
    serialPort = searchSerialPort (port);
    serialPort.addEventListener (this);
    serialPort.setSerialPortParams (speed, serialPort.DATABITS_8, serialPort.STOPBITS_1, serialPort.PARITY_NONE);
    serialOutput = serialPort.getOutputStream ();
    serialInput = serialPort.getInputStream ();
    serialPort.notifyOnDataAvailable (true);
    }
    catch(Exception e)
    {
     System.out.println ("Creating Terminal:\n  " + e);
    }
  }

  public static void main (String argv[]) throws IOException
  {
    String comport = null;
    String uploadname = null;
    int comspeed = 38400;

    for (int i = 0; i < argv.length; i++)
      {
	if (argv[i].equalsIgnoreCase ("-port") && i + 1 < argv.length)
	  comport = argv[++i];
	else if (argv[i].equalsIgnoreCase ("-upload") && i + 1 < argv.length)
	  uploadname = argv[++i];
	else if (argv[i].equalsIgnoreCase ("-help"))
	  usage ();
	else if (argv[i].equalsIgnoreCase ("-speed") && i + 1 < argv.length)
	  {
	    try
	    {
	      comspeed = Integer.parseInt (argv[++i]);
	    }
	    catch (NumberFormatException nfe)
	    {
	      System.out.println ("Setting Serial Port Speed:\n  " + nfe);
	    }
	    System.out.println ("Communication speed set to " + comspeed + " bps.");
	  }
      }
    Terminal terminal = new Terminal ((comport != null) ? comport : "COM1", comspeed);
    terminal.openTerminalWindow ();
    terminal.instruments = new InstrumentPanel ();
    if (uploadname != null)
      new upLoad (uploadname, terminal.serialOutput);
  }
}
