/*
 * AnalyzePanel.java
 * June 1998, Jean-Marc Koller
 */

package KheperaTerminal;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.BoxLayout;
import javax.swing.JSeparator;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.border.TitledBorder;
import javax.swing.border.LineBorder;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.comm.SerialPort;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.awt.FileDialog;

public class AnalyzePanel implements WindowListener, ActionListener, ChangeListener
{
  Analyzer[] analyzers;
  JFrame frame;
  JPanel panel;
  JScrollPane scrollpane;
  JSlider sliderT, sliderV;
  JTextField textT, textV;
  JButton button_export;

  int mintime, maxtime;
  boolean mintimevalid;

    byte[] ebuffer;
  int esize;
  int maxusedh;
  int startIndex;

  public AnalyzePanel (Instrument[]instruments, SerialPort sp)
  {
    System.out.println ("Starting AnalyzePanel...");
    ebuffer = null;
    esize = 0;
    downloader downl = new downloader (sp);
      ebuffer = downl.getBuffer ();
      esize = downl.getLength ();
      mintimevalid = false;
      maxusedh = 0;
    if (esize == 0)
      {
	System.out.println ("AnalyzePanel : Nothing to analyze!");
	return;
      }
    analyzers = new Analyzer[InstrumentPanel.MAXINSTR];
      analyzers[InstrumentPanel.MAXINSTR - 1] = new Xscale ();
    int index;
    for (index = 0; index < InstrumentPanel.MAXINSTR; index++)
      if (instruments[index] != null)
	{
	  analyzers[index] = instruments[index].createAnalyzer ();
	  maxusedh = index;
	}
    getStartOffset ();
    System.out.println ("Start offset is " + startIndex);
    if (startIndex == -1)
      return;
    allocateSpace ();
    parseData ();
    setTime (mintime, maxtime);
    frame = new JFrame ("Analyzer");
    frame.addWindowListener (this);
    JPanel Tpanel = new JPanel ();
    Tpanel.setBorder (new TitledBorder (LineBorder.createGrayLineBorder (), "Time factor"));
    sliderT = new JSlider (JSlider.HORIZONTAL, 0, 100, 10);
    sliderT.setMajorTickSpacing (20);
    sliderT.setMinorTickSpacing (5);
    sliderT.setSnapToTicks (true);
    sliderT.setPaintLabels (true);
    sliderT.setPaintTicks (true);
    sliderT.addChangeListener (this);
    textT = new JTextField ("" + sliderT.getValue (), 5);
    textT.addActionListener (this);
    Tpanel.add (textT);
    Tpanel.add (sliderT);
    JPanel Vpanel = new JPanel ();
    Vpanel.setBorder (new TitledBorder (LineBorder.createGrayLineBorder (), "Value factor"));
    sliderV = new JSlider (JSlider.HORIZONTAL, 0, 1000, 10);
    sliderV.setMajorTickSpacing (200);
    sliderV.setMinorTickSpacing (50);
    sliderV.setSnapToTicks (true);
    sliderV.setPaintLabels (true);
    sliderV.setPaintTicks (true);
    sliderV.addChangeListener (this);
    textV = new JTextField ("" + sliderV.getValue (), 5);
    textV.addActionListener (this);
    Vpanel.add (textV);
    Vpanel.add (sliderV);
    button_export = new JButton ("EXPORT DATA");
    button_export.addActionListener (this);
    JPanel spanel = new JPanel ();
    spanel.add (Tpanel);
    spanel.add (Vpanel);
    spanel.add (button_export);
    frame.getContentPane ().add ("North", spanel);
    panel = new JPanel ();
    panel.setLayout (new BoxLayout (panel, BoxLayout.Y_AXIS));
    for (index = 0; index < InstrumentPanel.MAXINSTR; index++)
      {
	Analyzer pa = analyzers[index];
	if (pa != null)
	  {
	    pa.addComponent (panel);
	    if (index < InstrumentPanel.MAXINSTR - 1)
	      panel.add (new JSeparator ());
	  }
      }
    scrollpane = new JScrollPane ();
    scrollpane.getViewport ().add (panel);
    frame.getContentPane ().add ("Center", scrollpane);
    setZoomTime (sliderT.getValue ());
    setZoomValue (sliderV.getValue ());
    frame.pack ();
    frame.show ();
  }

  private void parseData ()
  {
    int timestamp = 0;
    for (int index = startIndex; index < esize;)
      {
	if (ebuffer[index] == '@')
	  {
	    Analyzer pa = analyzers[ebuffer[index + 1]];
	    if (pa == null)
	      {
		System.out.println ("No analyzer at " + index);
		break;
	      }
	    index += 2;
	    timestamp = IntegerInstrument.byteToInt32 (ebuffer, index);
	    if (!mintimevalid)
	      {
		mintimevalid = true;
		mintime = timestamp;
	      }
	    index += 4;
	    index += pa.addData (timestamp, ebuffer, index);
	  }
	else
	  {
	    System.out.println ("out of phase!");
	    break;
	  }
      }
    maxtime = timestamp;
  }

  void exportData (String filename)
  {
    FileOutputStream fo;
      try
    {
      fo = new FileOutputStream (filename);
      for (int index = startIndex; index < esize;)
	{
	  if (ebuffer[index] == '@')
	    {
	      int chandler = ebuffer[index + 1];
	        index += 2;
	      int timestamp = IntegerInstrument.byteToInt32 (ebuffer, index);
	        index += 4;
	        fo.write (new String (timestamp + "\t").getBytes ());
	      for (int sh = 0; sh <= maxusedh; sh++)
		{
		  Analyzer pa = analyzers[sh];
		  if (pa != null)
		    {
		      if (sh == chandler)
			index += pa.exportData (fo, ebuffer, index);
		      else
			pa.exportData (fo, null, index);
		    }
		  fo.write ('\t');
		}
	      fo.write ('\r');
	      fo.write ('\n');
	    }
	  else
	    index++;
	}
      fo.close ();
    }
    catch (IOException ioe)
    {
      System.out.println ("Error during data exportation...");
    }
  }

  void getStartOffset ()
  {
    int index;
    for (index = 0; index < esize; index++)
      {
	if (ebuffer[index] == '@')
	  {
	    Analyzer p = analyzers[ebuffer[index + 1]];
	    if (p != null)
	      {
		int i2 = index + 1 + 4 + p.DataSize () + 1;
		if (ebuffer[i2] == '@')
		  {
		    startIndex = index;
		    return;
		  }
	      }
	  }
      }
    System.out.println ("unable to find startpoint in eventbuffer");
    startIndex = -1;
    return;
  }

  void allocateSpace ()
  {
    for (int index = startIndex; index < esize;)
      {
	if (ebuffer[index] == '@')
	  {
	    Analyzer pa = analyzers[ebuffer[index + 1]];
	    if (pa == null)
	      {
		System.out.println ("No analyzer at " + index);
		break;
	      }
	    index += pa.DataSize () + 2 + 4;
	    pa.incSize ();
	  }
	else
	  {
	    System.out.println ("out of phase!");
	    break;
	  }
      }
  }

  public void setZoomTime (int factor)
  {
    for (int index = 0; index < InstrumentPanel.MAXINSTR; index++)
      {
	Analyzer pa = analyzers[index];
	if (pa != null)
	    pa.setZoomTime (factor);
      }
  }

  void setZoomValue (int factor)
  {
    for (int index = 0; index < InstrumentPanel.MAXINSTR; index++)
      {
	Analyzer pa = analyzers[index];
	if (pa != null)
	    pa.setZoomValue (factor);
      }
  }

  public void setTime (int mint, int maxt)
  {
    for (int index = 0; index < InstrumentPanel.MAXINSTR; index++)
      {
	Analyzer pa = analyzers[index];
	if (pa != null)
	    pa.setTime (mint, maxt);
      }
  }

  public void stateChanged (ChangeEvent e)
  {
    if (e.getSource () == sliderT)
      {
	if (sliderT.getValue () < 1)
	  sliderT.setValue (1);
	textT.setText ("" + sliderT.getValue ());
      }
    else if (e.getSource () == sliderV)
      {
	if (sliderV.getValue () < 1)
	  sliderV.setValue (1);
	textV.setText ("" + sliderV.getValue ());
      }
    else
      return;
    if (sliderT.getValueIsAdjusting () || sliderV.getValueIsAdjusting ())
      return;
    setZoomTime (sliderT.getValue ());
    setZoomValue (sliderV.getValue ());
    panel.validate ();
    scrollpane.revalidate ();
  }

  public void actionPerformed (ActionEvent e)
  {
    try
    {
      if (e.getSource () == textT)
	{
	  int v = Integer.parseInt (textT.getText (), 10);
	  if (v < 1)
	    {
	      v = 1;
	      textT.setText ("" + v);
	    }
	  sliderT.setValue (v);
	}
      else if (e.getSource () == textV)
	{
	  int v = Integer.parseInt (textV.getText (), 10);
	  if (v < 1)
	    {
	      v = 1;
	      textV.setText ("" + v);
	    }
	  sliderV.setValue (v);
	}
      else if (e.getSource () == button_export)
	{
	  FileDialog uf = new FileDialog (frame, "Export data...");
	  uf.setMode (FileDialog.SAVE);
	  uf.show ();
	  if (uf.getFile () != null)
	    exportData (uf.getFile ());
	  return;
	}
    }
    catch (NumberFormatException nfe)
    {
      System.out.println ("Illegal number!");
      return;
    }
    setZoomTime (sliderT.getValue ());
    setZoomValue (sliderV.getValue ());
    //panel.validate();
    scrollpane.revalidate ();
    //frame.validate();
  }

  public void windowClosed (WindowEvent e)
  {
  }
  public void windowDeiconified (WindowEvent e)
  {
  }
  public void windowIconified (WindowEvent e)
  {
  }
  public void windowActivated (WindowEvent e)
  {
  }
  public void windowDeactivated (WindowEvent e)
  {
  }
  public void windowOpened (WindowEvent e)
  {
  }
  public void windowClosing (WindowEvent e)
  {
    //      System.exit(0);
  }

  class downloader
  {
    private final int MAXELENGTH = 64 * 1024;
      byte[] buffer = null;
    int elength = 0;

    public downloader (SerialPort sp)
    {
      InputStream is;
      OutputStream os;
      String toFreeze = "@IfReEzeK@IspitK";
        try
      {
	is = sp.getInputStream ();
	os = sp.getOutputStream ();
	os.write (toFreeze.getBytes ());
      }
      catch (IOException ioe)
      {
	return;
      }
      sp.notifyOnDataAvailable (false);
      buffer = new byte[MAXELENGTH];

      try
      {
	while (is.available () > 0)
	  if (is.read () == '@')
	    break;
	while (is.available () < 1);
	if (is.read () != 'I')
	  throw new IOException ("I");
	while (is.available () < 1);
	if (is.read () != 'x')
	  throw new IOException ("x");
	byte[]blenbuffer = new byte[4];
	while (is.available () < 4);
	is.read (blenbuffer);
	int blength = IntegerInstrument.byteToInt32 (blenbuffer, 0);
	if (blength > MAXELENGTH)
	  throw new IOException ("len>>");
	while (is.available () < 1);
	if (is.read () != 'J')
	  throw new IOException ("J");
	System.out.println ("Receiving a " + blength + " bytes eventbuffer...");
	int torec = blength;
	int pos = 0;
	while (torec > 0)
	  {
	    int nbd = is.read (buffer, pos, torec);
	    if (nbd == -1)
	      break;
	    torec -= nbd;
	    pos += nbd;
	  }
	while (is.available () < 1);
	if (is.read () != 'K')
	  throw new IOException ("K");
	elength = blength;
      }
      catch (IOException ioe)
      {
	System.out.println ("Error occured during reception of event buffer...");
	System.out.println (ioe);
      }
      sp.notifyOnDataAvailable (true);
      esize = elength;
      ebuffer = buffer;
    }

    public byte[] getBuffer ()
    {
      return buffer;
    }

    public int getLength ()
    {
      return elength;
    }

  };

};
