/*
 * ArrayInstrument.java
 * June 1998, Jean-Marc Koller
 */

package KheperaTerminal;

import javax.swing.JPanel;
import javax.swing.border.TitledBorder;
import javax.swing.border.LineBorder;

import java.awt.Component;
import java.awt.Graphics;
import javax.swing.JSeparator;
import java.io.FileOutputStream;
import java.io.IOException;

public class ArrayInstrument implements Instrument
{
  Instrument[] instruments;
  int nbinstruments;
  JPanel ipanel;
  int tsize;
  int mode;
  int[] instrOffset;
  String pname;

  public ArrayInstrument (JPanel panel, String name, int nbparams, byte[]params)
  {
    pname = name;
    int min = 0;
    int max = 0;
    if (nbparams < 1)
      {
	System.out.println ("Not enough parameters for ArrayInstrument! ");
	nbinstruments = 0;
	return;
      }
    int pindex = 0;
      mode = IntegerInstrument.byteToInt8 (params, pindex++);
      nbinstruments = IntegerInstrument.byteToInt8 (params, pindex++);
    if (nbparams < 1 + 1 + 2 * nbinstruments)
      {
	System.out.println ("ArrayInstrument: Not enough parameters for " + nbinstruments + " sub-instruments!");
	nbinstruments = 0;
	return;
      }
    if ((mode & IntegerInstrument.MODE_MINMAX) != 0)
      {
	min = IntegerInstrument.byteToInt32 (params, pindex);
	pindex += 4;
	max = IntegerInstrument.byteToInt32 (params, pindex);
	pindex += 4;
	//System.out.println("min="+min+" max="+max );
      }
    System.out.println ("Creating ArrayInstrument for " + nbinstruments + " instruments.");
    instruments = new Instrument[nbinstruments];
    instrOffset = new int[nbinstruments];
    tsize = 0;
    ipanel = new JPanel ();
    ipanel.setBorder (new TitledBorder (LineBorder.createGrayLineBorder (), name));
    for (int i = 0; i < nbinstruments; i++)
      {
	int size = params[pindex++];
	String iname = new String (params, pindex++, 1);
	if ((mode & IntegerInstrument.MODE_MINMAX) != 0)
	  instruments[i] = new IntegerInstrument (ipanel, iname, (size < 0), (size < 0) ? -size : size, true, mode, min, max);
	else
	  instruments[i] = new IntegerInstrument (ipanel, iname, (size < 0), (size < 0) ? -size : size, true, mode);
	instrOffset[i] = tsize;
	tsize += instruments[i].DataSize ();
      }
    panel.add (ipanel);
  }

  public void update (byte[]data, int offset)
  {
    Instrument instr;
    for (int i = 0; i < nbinstruments; i++)
      {
	instr = instruments[i];
	instr.update (data, instrOffset[i]);
      }
  }

  public void performUpdateObject ()
  {
    for (int i = 0; i < nbinstruments; i++)
      instruments[i].performUpdateObject ();
  }

  public int DataSize ()
  {
    return tsize;
  }

  public Analyzer createAnalyzer ()
  {
    return new ArrayInstrument.analyzer ();
  }

  protected class analyzer extends Component implements Analyzer
  {
    Analyzer[] subAnalyzers;

    analyzer ()
    {
      subAnalyzers = new Analyzer[nbinstruments];
      Instrument instr;
      for (int i = 0; i < nbinstruments; i++)
	  subAnalyzers[i] = instruments[i].createAnalyzer ();
    }

    public int addData (int timestamp, byte[]buffer, int offset)
    {
      for (int i = 0; i < nbinstruments; i++)
	offset += subAnalyzers[i].addData (timestamp, buffer, offset);
      return tsize;
    }

    public void setTime (int mintime, int maxtime)
    {
      for (int i = 0; i < nbinstruments; i++)
	subAnalyzers[i].setTime (mintime, maxtime);
    }

    public int exportData (FileOutputStream fo, byte[]buffer, int offset) throws IOException
    {
      for (int i = 0; i < nbinstruments; i++)
	{
	  if (buffer != null)
	    offset += subAnalyzers[i].exportData (fo, buffer, offset);
	  if (i <= nbinstruments)
	    fo.write ('\t');
	}
      return tsize;
    }

    public int DataSize ()
    {
      return tsize;
    }

    public Component getComponent ()
    {
      return this;
    }

    public void incSize ()
    {
      for (int i = 0; i < nbinstruments; i++)
	subAnalyzers[i].incSize ();
    }

    public String getName ()
    {
      return "Array " + pname;
    }

    public void paint (Graphics g)
    {
    }

    public void addComponent (JPanel panel)
    {
      for (int i = 0; i < nbinstruments; i++)
	{
	  subAnalyzers[i].addComponent (panel);
	  if (i < nbinstruments - 1)
	    panel.add (new JSeparator ());
	}
    }

    public void setZoomTime (int factor)
    {
      for (int i = 0; i < nbinstruments; i++)
	subAnalyzers[i].setZoomTime (factor);
    }

    public void setZoomValue (int factor)
    {
      for (int i = 0; i < nbinstruments; i++)
	subAnalyzers[i].setZoomValue (factor);
    }

  };

};
