/*
 * BooleanInstrument.java
 * June 1998, Jean-Marc Koller
 */

package KheperaTerminal;

import javax.swing.JPanel;
import javax.swing.JCheckBox;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.Color;
import java.io.FileOutputStream;
import java.io.IOException;

public class BooleanInstrument implements Instrument, ActionListener
{
  JCheckBox checkbox;
  boolean curvalue;
  boolean toUpdate = false;
  String tname;

  public BooleanInstrument (JPanel panel, String pname, int nbparams, byte[]params)
  {
    tname = pname;
    System.out.println ("BooleanInstrument construction...");
    checkbox = new JCheckBox (tname);
    checkbox.setFocusPainted (false);
    checkbox.addActionListener (this);
    panel.add (checkbox);
  }

  public void update (byte[]data, int offset)
  {
    curvalue = (data[offset] != 0);
    toUpdate = true;
  }

  public void actionPerformed (ActionEvent e)
  {
    checkbox.setSelected (curvalue);
    toUpdate = false;
  }

  public void performUpdateObject ()
  {
    if (!toUpdate)
      return;
    checkbox.setSelected (curvalue);
    toUpdate = false;
  }

  public int DataSize ()
  {
    return 1;
  }

  public Analyzer createAnalyzer ()
  {
    return new BooleanInstrument.analyzer ();
  }

  protected class analyzer extends Component implements Analyzer
  {
    int nbValues = 0;
    int[] timestamps = null;
      boolean[] values = null;
    int zoomT = 1;
    int timeoffset;

      analyzer ()
    {
    }

    public void setTime (int mintime, int maxtime)
    {
      timeoffset = mintime;
    }

    int curI;
    public int addData (int timestamp, byte[]buffer, int offset)
    {
      if (timestamps == null)
	{
	  timestamps = new int[nbValues];
	    values = new boolean[nbValues];
	    curI = 0;
	}
      timestamps[curI] = timestamp;
        values[curI] = (buffer[offset] != 0);
        curI++;
        return 1;
    }

    public int exportData (FileOutputStream fo, byte[]buffer, int offset) throws IOException
    {
      if (buffer != null)
	fo.write (((buffer[offset] != 0) ? '1' : '0'));
      return 1;
    }

    public int DataSize ()
    {
      return 1;
    }

    public Component getComponent ()
    {
      return this;
    }

    public void incSize ()
    {
      nbValues++;
    }

    public String getName ()
    {
      return "boolean " + tname;
    }

    public void addComponent (JPanel panel)
    {
      panel.add (this);
    }

    public void setZoomTime (int factor)
    {
      zoomT = factor;
    }

    public void setZoomValue (int factor)
    {
    }

    public Dimension getMinimumSize ()
    {
      if (timestamps == null)
	return new Dimension (0, 0);
      else
	return new Dimension ((timestamps[nbValues - 1] - timeoffset) / zoomT, 16);
    }

    public void paint (Graphics g)
    {
      for (int i = 0; i < nbValues - 1; i++)
	{
	  int x1 = (timestamps[i] - timeoffset) / zoomT;
	  int x2 = (timestamps[i + 1] - timeoffset) / zoomT;
	  if (values[i])
	    {
	      g.setColor (Color.red);
	      g.fillRect (x1, 0, x2 - x1, 15);
	    }
	  else
	    {
	      g.setColor (Color.green);
	      g.drawLine (x1, 15, x2, 15);
	    }
	}
    }

  };

};
