/*
 * InstrumentPanel.java
 * June 1998, Jean-Marc Koller
 */

package KheperaTerminal;

import javax.swing.JFrame;
import javax.swing.JPanel;
import java.awt.FlowLayout;
import javax.swing.JScrollPane;
import javax.swing.border.TitledBorder;
import java.awt.Component;
import java.awt.AWTEvent;
import java.awt.Dimension;
import java.awt.Toolkit;


public class InstrumentPanel extends Thread
{
  JFrame frame, actorFrame;
  public static final int MAXINSTR = 256;
  JScrollPane scrollpane, actorScrollpane;

  JPanel panel, actorPanel;

  private Instrument[] instruments;

  public Instrument[] getInstrumentArray ()
  {
    return instruments;
  }

  public InstrumentPanel ()
  {
    curBuffer = new byte[maxBuffer];
    instruments = new Instrument[MAXINSTR];
    System.out.println ("Starting InstrumentPanel...");
    frame = null;
    panel = null;
    this.start ();
  }

  void openInstrumentPanel ()
  {
    if (frame != null)
      return;
    frame = new JFrame ("Instruments panel");
    frame.getContentPane ().setLayout (new FlowLayout ());
    panel = new JPanel ();
    scrollpane = new JScrollPane ();
    scrollpane.getViewport ().add (panel);
    frame.getContentPane ().add ("Center", scrollpane);
    frame.pack ();
    frame.show ();
  }

  void updateInstrumentPanel ()
  {
    scrollpane.validate ();
    frame.validate ();
    frame.pack ();
  }

  void openActorPanel ()
  {
    if (actorFrame != null)
      return;
    actorFrame = new JFrame ("Actors panel");
    actorFrame.getContentPane ().setLayout (new FlowLayout ());
    actorPanel = new JPanel ();
    actorScrollpane = new JScrollPane ();
    actorScrollpane.getViewport ().add (actorPanel);
    actorFrame.getContentPane ().add ("Center", actorScrollpane);
    actorFrame.pack ();
    actorFrame.setLocation (0, Toolkit.getDefaultToolkit ().getScreenSize ().height / 2);
    actorFrame.show ();
  }

  void updateActorPanel ()
  {
    actorScrollpane.validate ();
    actorFrame.validate ();
    actorFrame.pack ();
  }

  private int HandleState = 0;
  private byte curHandler = 0;
  private byte curInstrument = 0;
  private byte[] curBuffer = null;
  private int curIndex = 0;
  private final static int maxBuffer = 64;
  private String curName = null;
  private int nbParams = 0;
  public int HandleData (byte[]in, int totalLength)
  {
    for (int i = 0; i < totalLength; i++)
      {
	byte cv = in[i];
	switch (HandleState)
	  {
	  case 0:
	    if (cv == '@')
	      HandleState = 1;
	    break;
	    case 1:
	    if (cv == 'I')
	      HandleState = 2;
	    else
	      HandleState = 0;
	    break;
	    case 2:
	    if (cv == 'a')
	      HandleState = 3;
	    else if (cv == 'n')
	      HandleState = 11;
	    else
	      HandleState = 0;
	    break;
	    case 3:		/* New Instrument */
	    curHandler = cv;
	    HandleState++;
	    break;
	    case 4:
	    curInstrument = cv;
	    HandleState++;
	    curIndex = 0;
	    break;
	    case 5:
	    HandleState = (cv == 'J') ? 6 : 0;
	    break;
	    case 6:
	    if (cv == 0)
	      {
		curName = new String (curBuffer, 0, curIndex);
		HandleState++;
		curIndex = 0;
		break;
	      }
	    if (curIndex > maxBuffer)
	      {
		HandleState = 0;
		break;
	      }
	    curBuffer[curIndex++] = cv;
	    break;
	  case 7:
	    HandleState = (cv == 'M') ? 8 : 0;
	    if (HandleState == 0)
	      System.out.println ("Framing error M.");
	    break;
	  case 8:
	    nbParams = cv;
	    if (nbParams > maxBuffer)
	      {
		HandleState = 0;
		break;
	      }
	    HandleState = (nbParams == 0) ? 10 : 9;
	    break;
	  case 9:
	    curBuffer[curIndex++] = cv;
	    if (curIndex >= nbParams)
	      HandleState++;
	    break;
	  case 10:
	    if (cv == 'K')
	      {
		createNewInstrument ();
	      }
	    else
	      System.out.println ("Framing error at end.");
	    HandleState = 0;
	    break;
	  case 11:		/* update Instrument */
	    curHandler = cv;
	    if (instruments[curHandler] != null)
	      {
		nbParams = instruments[curHandler].DataSize ();
		curIndex = 0;
		HandleState = (nbParams > 0) ? 12 : 13;
	      }
	    else
	      HandleState = 0;
	    break;
	  case 12:
	    curBuffer[curIndex++] = cv;
	    if (curIndex >= nbParams)
	      HandleState++;
	    break;
	  case 13:
	    if (cv == 'K')
	      instruments[curHandler].update (curBuffer, 0);
	    HandleState = 0;
	    break;
	  default:
	    HandleState = 0;
	    break;
	  }
      }
    return 0;
  }

  private void createNewInstrument ()
  {
    if (instruments[curHandler] != null)
      {
	System.out.println ("Instrument " + curHandler + " has not been released! Not allocating new instrument!");
      }
    else
      {
	switch (curInstrument)
	  {
	  case 0:
	    openInstrumentPanel ();
	    instruments[curHandler] = new BooleanInstrument (panel, curName, nbParams, curBuffer);
	    updateInstrumentPanel ();
	    break;
	  case 1:
	    openInstrumentPanel ();
	    instruments[curHandler] = new IntegerInstrument (panel, curName, nbParams, curBuffer);
	    updateInstrumentPanel ();
	    break;
	  case 2:
	    openInstrumentPanel ();
	    instruments[curHandler] = new ArrayInstrument (panel, curName, nbParams, curBuffer);
	    updateInstrumentPanel ();
	    break;
	  case 3:
	    openActorPanel ();
	    instruments[curHandler] = new ButtonInstrument (actorPanel, curName, nbParams, curBuffer, curHandler);
	    updateActorPanel ();
	    break;
	  case 4:
	    openActorPanel ();
	    instruments[curHandler] = new IntegerInstrument (actorPanel, curName, nbParams, curBuffer, curHandler);
	    updateActorPanel ();
	    break;
	  default:
	    return;
	  }
      }
  }

  /* Thread! Updates the content of the instruments. */
  public void run ()
  {
    Instrument instr;
    for (;;)
      {
	for (int i = 0; i < MAXINSTR; i++)
	  {
	    instr = instruments[i];
	    if (instr != null)
	      instr.performUpdateObject ();
	  }
	try
	{
	  sleep (80);
	}
	catch (InterruptedException ie)
	{
	  return;
	}
      }
  }
};
