/*
 * IntegerInstrument.java
 * June 1998, Jean-Marc Koller
 */

package KheperaTerminal;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import javax.swing.border.TitledBorder;
import javax.swing.border.LineBorder;
import javax.swing.JTextField;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.Color;
import java.io.FileOutputStream;
import java.io.IOException;

public class IntegerInstrument implements Instrument, ChangeListener, ActionListener
{
	JSlider slider=null;
	JTextField textFieldD=null, textFieldH=null, textFieldB=null;
	JPanel ipanel;
	int size, minV, maxV, offset;
	boolean unsigned;
	int curvalue = 0;
	boolean toUpdate;
	int oldValue;
	boolean isInput = false;
	int handler = 0;
	String tname;

	static public int byteToInt8( byte[] p, int offset )
	{
		return ((p[offset]>=0)?p[offset]:(p[offset]+256));
	}

	static public int byteToInt32( byte[] p, int offset )
	{
		return (((p[offset]>=0)?p[offset]:(p[offset]+256))<<24) + (((p[1+offset]>=0)?p[1+offset]:(p[1+offset]+256))<<16) + (((p[2+offset]>=0)?p[2+offset]:(p[2+offset]+256))<<8) + ((p[3+offset]>=0)?p[3+offset]:(p[3+offset]+256));
	}

	public static final int MODE_SLIDER = 1<<0;
	public static final int MODE_DECIMAL = 1<<1;
	public static final int MODE_HEXA = 1<<2;
	public static final int MODE_BINARY = 1<<3;
	public static final int MODE_MINMAX = 1<<4;
	int mode;

	public IntegerInstrument( JPanel panel, String pname, int nbparams, byte[] params )
	{
		tname = pname;
		System.out.println("IntegerInstrument construction...");
		offset = 0;
		unsigned = true;
		if (nbparams<1) {
			System.out.println("Not enough parameter! Assuming default values!");
			size = 1;
			minV = 0;
			maxV = 255;
			mode = MODE_SLIDER|MODE_DECIMAL;
		} else {
			mode = params[0];
			size = params[1];
			if (size>16) {
				unsigned = false;
				size -= 16;
			}
			if (nbparams>=1+1+4)
				minV = byteToInt32( params, 2 );
			else
				minV = (unsigned) ? 0 : -(1<<(size*8-1));
			if (nbparams>=1+1+4+4)
				maxV = byteToInt32( params, 1+1+4 );
			else
				maxV = (unsigned) ? (1<<(size*8))-1 : (1<<(size*8-1))-1;
			if (nbparams>=1+1+4+4+4)
				offset = byteToInt32( params, 1+1+4+4 );
		}
		openPanels( panel, false );
	}

	// Constructor for IntegerInstrument as actor
	public IntegerInstrument( JPanel panel, String pname, int nbparams, byte[] params, int chandler )
	{
		tname = pname;
		System.out.println("IntegerInstrument as actor construction...");
		handler = chandler;
		offset = 0;
		size = 0;
		unsigned = true;
		if (nbparams>=2*4+1+4) {
			mode = params[0];
			curvalue = byteToInt32( params, 1 );
			minV = byteToInt32( params, 1+4 );
			maxV = byteToInt32( params, 1+4+4 );
			isInput = true;
		} else {
			System.out.println("Error: not enough parameter for IntegerInstrument as actor...");
			return;
		}
		openPanels( panel, false );
	}

	public IntegerInstrument( JPanel panel, String pname, boolean isUnsigned, int sizeVar )
	{
		tname = pname;
		size = sizeVar;
		unsigned = isUnsigned;
		minV = (unsigned) ? 0 : -(1<<(size*8-1));
		maxV = (unsigned) ? (1<<(size*8))-1 : (1<<(size*8-1))-1;
		mode = MODE_SLIDER|MODE_DECIMAL;
		offset = 0;
		openPanels( panel, false );
	}

	public IntegerInstrument( JPanel panel, String pname, boolean isUnsigned, int sizeVar, boolean isVertical, int cMode )
	{
		tname = pname;
		mode = cMode;
		size = sizeVar;
		unsigned = isUnsigned;
		minV = (unsigned) ? 0 : -(1<<(size*8-1));
		maxV = (unsigned) ? (1<<(size*8))-1 : (1<<(size*8-1))-1;
		offset = 0;
		openPanels( panel, isVertical );
	}

	public IntegerInstrument( JPanel panel, String pname, boolean isUnsigned, int sizeVar, boolean isVertical, int cMode, int min, int max )
	{
		tname = pname;
		mode = cMode;
		size = sizeVar;
		unsigned = isUnsigned;
		minV = min;
		maxV = max;
		offset = 0;
		openPanels( panel, isVertical );
	}

	public void stateChanged( ChangeEvent e )
	{
		if (!isInput) {
			if (slider.getValueIsAdjusting())
				slider.setValue( curvalue );
			return;
		}
		curvalue = slider.getValue();
		updateActor();
		toUpdate = true;
		performUpdateObject();
	}

	public void actionPerformed( ActionEvent e )
	{
		try {
			if (e.getSource()==textFieldD) {
				curvalue = Integer.parseInt( textFieldD.getText(), 10 );
			} else if (e.getSource()==textFieldH) {
				curvalue = Integer.parseInt( textFieldH.getText(), 16 );
			} else if (e.getSource()==textFieldB) {
				curvalue = Integer.parseInt( textFieldB.getText(), 2 );
			}
		} catch (NumberFormatException nfe) {
			System.out.println("Illegal number!");
			return;
		}
		updateActor();
		toUpdate = true;
		performUpdateObject();
	}

	void updateActor()
	{
		byte buffer[] = new byte[10];
		buffer[0] = (byte)'@';
		buffer[1] = (byte)'I';
		buffer[2] = (byte)'b';
		buffer[3] = (byte)handler;
		buffer[4] = (byte)4;
		buffer[5] = (byte)(curvalue>>24);
		buffer[6] = (byte)(curvalue>>16);
		buffer[7] = (byte)(curvalue>>8);
		buffer[8] = (byte)curvalue;
		buffer[9] = (byte)'K';

		/* terminal.sendData( buffer ); */
	}

	private void openPanels( JPanel panel, boolean isVertical )
	{
		toUpdate = false;
		oldValue = -1;
		System.out.println("size="+size+" minV="+minV+" maxV="+maxV+" offset="+offset);
		slider = null;
		ipanel = new JPanel();
		if (isVertical)
			ipanel.setLayout( new BorderLayout() );
		ipanel.setBorder( new TitledBorder( LineBorder.createGrayLineBorder(), tname ) );
		if ((mode & MODE_DECIMAL)!=0) {
			textFieldD = new JTextField( 4 );
			textFieldD.setEditable( isInput );
			if (isInput)
				textFieldD.addActionListener( this );
		}
		if ((mode & MODE_HEXA)!=0) {
			textFieldH = new JTextField( 4 );
			textFieldH.setEditable( isInput );
			if (isInput)
				textFieldH.addActionListener( this );
		}
		if ((mode & MODE_BINARY)!=0) {
			textFieldB = new JTextField( 4 );
			textFieldB.setEditable( isInput );
			if (isInput)
				textFieldB.addActionListener( this );
		}
		if ((mode & MODE_SLIDER)!=0) {
			if (curvalue<minV)
				curvalue = minV;
			slider = new JSlider( JSlider.HORIZONTAL, minV, maxV, curvalue );
			slider.addChangeListener( this );
			slider.setOrientation( (isVertical) ? JSlider.VERTICAL : JSlider.HORIZONTAL );
			switch(size) {
			case 0:
				int delta = maxV-minV;
				long p10 = Math.round( Math.log(delta)/Math.log(10) );
				if (p10>=2) {
					slider.setMajorTickSpacing( 5*(int)Math.round( Math.pow( 10, p10-1 )) );
					slider.setMinorTickSpacing( (int)Math.round( Math.pow( 10, p10-2 )) );
					slider.setPaintTicks( true );
					slider.setPaintLabels( true );					
				}
				break;
			case 1:
				slider.setMajorTickSpacing( 50 );
				slider.setMinorTickSpacing( 10 );
				slider.setPaintTicks( true );
				slider.setPaintLabels( true );
				break;
			case 2:
				slider.setMajorTickSpacing( (maxV-minV>10000) ? 10000 : 1000 );
				slider.setMinorTickSpacing( (maxV-minV>10000) ? 2000 : 100 );
				slider.setPaintTicks( true );
				slider.setPaintLabels( false );
				break;
			case 4:
				slider.setMajorTickSpacing( 500000000 );
				slider.setPaintTicks( true );
				slider.setPaintLabels( false );
				break;
			case 8:
				break;
			}
		}
		if (isVertical) {
			if (slider!=null)
				ipanel.add( slider, BorderLayout.NORTH );
			if (textFieldD!=null)
				ipanel.add( textFieldD, BorderLayout.SOUTH );
			if (textFieldH!=null)
				ipanel.add( textFieldH, BorderLayout.SOUTH );
			if (textFieldB!=null)
				ipanel.add( textFieldB, BorderLayout.SOUTH );
		} else {
			if (textFieldD!=null)
				ipanel.add( textFieldD );
			if (textFieldH!=null)
				ipanel.add( textFieldH );
			if (textFieldB!=null)
				ipanel.add( textFieldB );
			if (slider!=null)
				ipanel.add( slider );
		}
		panel.add( ipanel );
		if (isInput) {
			toUpdate = true;
			performUpdateObject();
		}
	}


	public void update( byte[] data, int doffset )
	{
		int value = 0;
		int tsize = size, i=doffset;
		for( ;tsize>0; tsize--, i++ ) {
			value *= 256;
			value += (!unsigned || data[i]>=0) ? offset+data[i] : offset+data[i]+256;
		}
		curvalue = value;
		toUpdate = true;
	}

	public void performUpdateObject()
	{
		if (!toUpdate)
			return;
		if (oldValue==curvalue)
			return;
		oldValue = curvalue;
		if (slider!=null)
			slider.setValue( curvalue );
		if (textFieldD!=null)
			textFieldD.setText( ""+curvalue );
		if (textFieldH!=null)
			textFieldH.setText( Integer.toHexString(curvalue) );
		if (textFieldB!=null)
			textFieldB.setText( Integer.toBinaryString(curvalue) );
		toUpdate = false;
	}

	public int DataSize()
	{
		return size;
	}

	public Analyzer	createAnalyzer()
	{
		return new IntegerInstrument.analyzer();
	}

protected class analyzer extends Component implements Analyzer
{
	int nbValues = 0;
	int[] timestamps = null;
	int[] values = null;
	int zoomT = 1;
	int zoomV = 1;
	int timeoffset;

	analyzer()
	{
	}

	public void setTime( int mintime, int maxtime )
	{
		timeoffset = mintime;
	}

	int curI;
	public int addData( int timestamp, byte[] buffer, int doffset )
	{
		if (timestamps==null) {
			timestamps = new int[nbValues];
			values = new int[nbValues];
			curI = 0;
		}
		int value = 0;
		int tsize = size, i=doffset;
		for( ;tsize>0; tsize--, i++ ) {
			value *= 256;
			value += (!unsigned || buffer[i]>=0) ? offset+buffer[i] : offset+buffer[i]+256;
		}
		timestamps[curI] = timestamp;
		values[curI] = value;
		curI++;
		return size;
	}

	public int exportData( FileOutputStream fo, byte[] buffer, int doffset ) throws IOException
	{
		if (buffer==null)
			return size;
		int value = 0;
		int tsize = size, i=doffset;
		for( ;tsize>0; tsize--, i++ ) {
			value *= 256;
			value += (!unsigned || buffer[i]>=0) ? offset+buffer[i] : offset+buffer[i]+256;
		}
		String str = new String( ""+value );
		fo.write( str.getBytes() );
		return size;
	}

	public int DataSize()
	{
		return size;
	}

	public Component getComponent()
	{
		return this;
	}

	public void incSize()
	{
		nbValues++;
	}

	public String getName()
	{
		return "Integer "+tname;
	}

	public void addComponent( JPanel panel )
	{
		panel.add( this );
	}

	public Dimension getMinimumSize()
	{
		if (timestamps==null)
			return new Dimension( 0, 0 );
		else
			return new Dimension( (timestamps[nbValues-1]-timeoffset)/zoomT , (maxV-minV)/zoomV );
	}

	public void setZoomTime( int factor )
	{
		zoomT = factor;
	}

	public void setZoomValue( int factor )
	{
		zoomV = factor;
	}

	public Dimension getPreferredSize()
	{
		return getMinimumSize();
	}

	public void paint( Graphics g )
	{
		//System.out.println("name="+getName());
		//System.out.println("graphics.bounds="+g.getClipBounds());
		g.setColor( Color.darkGray );
		int y0 = (maxV-0-minV)/zoomV;
		g.drawLine( 0, y0, (timestamps[nbValues-1]-timeoffset)/zoomT, y0 );
		for( int i=1; i<nbValues; i++ ) {
			int x1 = (timestamps[i-1]-timeoffset)/zoomT;
			int x2 = (timestamps[i]-timeoffset)/zoomT;
			int y1 = (maxV-values[i-1]-offset-minV)/zoomV;
			int y2 = (maxV-values[i]-offset-minV)/zoomV;
			g.setColor( Color.red );
			if (i==1)
				g.drawOval( x1-2,y1-2, 4,4 );
			g.drawOval( x2-2,y2-2, 4,4 );
			g.setColor( Color.blue );
			g.drawLine( x1,y1 ,x2,y2 );
		}
	}
};

};
