/*
   ; K-TEAM Terminal Program
   ; =======================

   ;------------------------------------------------------------------------
   ; Author:    Jean Marc Koller 06/98
   ; Modifs:    Olivier Carmona  07/99
   ;
   ; Project:   KHEPERA
   ; Goal:              Source file of the K-TEAM Terminal
   ;
   ;
   ;------------------------------------------------------------------------
 */

/* Main programm of kheperaTerminal */

package KheperaTerminal;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.util.Enumeration;
import javax.comm.SerialPort;
import javax.comm.PortInUseException;
import javax.comm.CommPortIdentifier;
import javax.comm.SerialPortEvent;
import javax.comm.SerialPortEventListener;
import javax.comm.UnsupportedCommOperationException;
import javax.comm.NoSuchPortException;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JToggleButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JLabel;
import javax.swing.UIManager;
import java.awt.FileDialog;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JTextArea;
//import java.awt.TextArea;
//import java.awt.Color;
import javax.swing.text.*;
import javax.swing.KeyStroke;
import javax.swing.Action;


public class Main implements KeyListener, 
WindowListener, SerialPortEventListener, ActionListener
{
  SerialPort serialPort;
  OutputStream serialOutput;
  InputStream serialInput;
  JFrame terminalFrame;
  JPanel but_panel;
  JToggleButton but_freeze, but_analyze;
  JTextArea terminalText;
  JScrollPane scrollpane;
//  TextArea terminalText;

  InstrumentPanel instruments;

  void createMenu (JFrame frame)
  {
    JMenuBar mbar = new JMenuBar ();
    JMenu menu = new JMenu ("File");
    JMenuItem item;
      menu.add (item = new JMenuItem ("Upload file"));
      item.addActionListener (this);
      menu.addSeparator ();
      menu.add (item = new JMenuItem ("Exit"));
      item.addActionListener (this);
      mbar.add (menu);
      menu = new JMenu ("?");
      menu.add (item = new JMenuItem ("About..."));
      item.addActionListener (this);
      mbar.add (menu);
      frame.setJMenuBar (mbar);
  }

  void openTerminalWindow ()
  {
    terminalFrame = new JFrame ("Terminal");
    terminalText = new JTextArea (25, 80);
//    terminalText = new TextArea ("", 25, 80, TextArea.SCROLLBARS_VERTICAL_ONLY);
//    terminalText.setBackground (Color.white);
    terminalText.addKeyListener (this);

    // Create a keymap that makes more sense on the Mac.
    Keymap keymap = terminalText.addKeymap("MacKeyBindings", terminalText.getKeymap());
    // Arrow key actions.
    keymap.addActionForKeyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 0), 0); // new LeftArrowAction());
    keymap.addActionForKeyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 0), 0); //new RightArrowAction());
    keymap.addActionForKeyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0), 0); //new UpArrowAction());
    keymap.addActionForKeyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0), 0); //new DownArrowAction());

    terminalText.setKeymap(keymap);

    // JTextArea do not manage scrolling bar: Add it
    scrollpane = new JScrollPane (terminalText);
    scrollpane.getViewport ().add (terminalText);

    createMenu (terminalFrame);
    but_panel = new JPanel ();
    but_freeze = new JToggleButton ("FREEZE");
    but_freeze.addActionListener (this);
    but_panel.add (but_freeze);
    but_analyze = new JToggleButton ("ANALYZE");
    but_analyze.addActionListener (this);
    but_panel.add (but_analyze);
    terminalFrame.getContentPane ().add ("North", but_panel);
    terminalFrame.getContentPane ().add ("Center", scrollpane);
//    terminalFrame.getContentPane ().add ("Center", terminalText);
    terminalFrame.addWindowListener (this);
    terminalFrame.pack ();
    terminalFrame.show ();
  }

  public void keyTyped (KeyEvent e)
  {
    try
     {
       serialOutput.write (e.getKeyChar ());
     }
    catch (IOException ioe)
     {
       System.out.println (ioe);
     }
  }
  public void keyReleased (KeyEvent e) { }
  public void keyPressed (KeyEvent e) { }

  public void windowClosed (WindowEvent e) { }
  public void windowDeiconified (WindowEvent e) { }
  public void windowIconified (WindowEvent e) { }
  public void windowActivated (WindowEvent e) { }
  public void windowDeactivated (WindowEvent e) { }
  public void windowOpened (WindowEvent e) { }
  public void windowClosing (WindowEvent e)
  {
    System.exit (0);
  }

  public void actionPerformed (ActionEvent e)
  {
    String action = e.getActionCommand ();
    if (action.equals ("Exit"))
        System.exit (0);
    if (action.equals ("Upload file"))
      {
	FileDialog uf = new FileDialog (terminalFrame, "Upload S to Khepera");
	  uf.setMode (FileDialog.LOAD);
	  uf.show ();
	if (uf.getFile () != null)
	  {
	    try
	    {
	      new upLoad (uf.getDirectory () + uf.getFile (), serialOutput);
	    }
	    catch (IOException ioe)
	    {
	    }
	  }
	return;
      }
    if (action.equals ("About..."))
      {
       JOptionPane pane = new JOptionPane(new JLabel( "Jean Marc Koller, EPFL 06/98\r\nOlivier Carmona, K-TEAM 07/99"));
       JDialog dialog = pane.createDialog(terminalFrame, "About KheperaTerminal..." );
       dialog.show();
       return;
      }
    if (action.equals ("FREEZE"))
      {
	String toFreeze = "@IfReEzeK";
	String toThaw = "@IthawK";
	sendData ((!but_freeze.isSelected ())? toThaw.getBytes () : toFreeze.getBytes ());
	sendData ((!but_freeze.isSelected ())? toThaw.getBytes () : toFreeze.getBytes ());
	return;
      }
    if (action.equals ("ANALYZE"))
      {
	AnalyzePanel ap = new AnalyzePanel (instruments.getInstrumentArray (), serialPort);
      }
    System.out.println ("action=" + e);
  }

  public void serialEvent (SerialPortEvent ev)
  {
    switch (ev.getEventType ())
      {
      case ev.DATA_AVAILABLE:
	try
	{
	  byte[]buffer = new byte[128];
	  do
	    {
	      int len = serialInput.read (buffer);
	      if (instruments != null)
		  instruments.HandleData (buffer, len);
	       displayCleanBuffer (new String (buffer, 0, len));
	    }
	  while (serialInput.available () > 0);
	}
	catch (IOException ioe)
	{
	  System.out.println (ioe);
	}
	break;
      }
  }

  private boolean persistantHide = false;

  public void displayCleanBuffer (String in)
  {
    if (terminalText == null)
      return;
    int begin = 0;
    int sindex = (persistantHide) ? 0 : in.indexOf ("@I");
    if (sindex != -1)
      {
	int eindex = in.indexOf ('K');
	if (eindex != -1)
	  {
	    StringBuffer ws = new StringBuffer (in.length ());
	    while (sindex != -1 && eindex != -1)
	      {
		if (persistantHide)
		  persistantHide = false;
		else
		  {
		    if (sindex > begin)
		      ws.append (in.substring (begin, sindex));
		  }
		begin = eindex + 1;
		sindex = in.indexOf ("@I", begin);
		eindex = (sindex == -1) ? -1 : in.indexOf ('K', sindex);
	      }
	    persistantHide = (sindex != -1 && eindex == -1);
	    if (sindex == -1 && !persistantHide)
	      ws.append (in.substring (begin));
	    terminalText.append (ws.toString ());
	    return;
	  }
	else
	  {
	    persistantHide = true;
	    return;
	  }
      }
    persistantHide = false;
    terminalText.append (in);
    /* if (scrollpane.getViewport ().scrollUnderway == false) 
      {
           scrollpane.setViewportView(in, pos);
       
      }
     */
  }

  public void sendData (byte[]data)
  {
    try
    {
      serialOutput.write (data);
    }
    catch (IOException ioe)
    {
      System.out.println ("Sending Data: "+ioe);
    }
  }

  private static void usage ()
  {
    System.out.println ("KheperaTerminal: 1998 Jean-Marc Koller / 1999 Olivier Carmona");
    System.out.println ("       -help           : this message");
    System.out.println ("usage: -port <comport> : specify a comport (default:\"COM1\")");
    System.out.println ("       -upload <file>  : uploads a Srecord file to Khepera");
    System.out.println ("       -speed <speed>  : specify a baudrate for the port (default: 38400)");
  }

  static SerialPort searchSerialPort (String port) throws NoSuchPortException, PortInUseException
  {
    Enumeration portList;
    CommPortIdentifier portId = null;
    
    portId = CommPortIdentifier.getPortIdentifier (port);
    System.out.println ("using " + portId.getName ());
    return (SerialPort) portId.open ("KheperaLoad", 400);
  }

  public Main (String port, int speed)
  {
    try
    {
    serialPort = searchSerialPort (port);
    serialPort.addEventListener (this);
    serialPort.setSerialPortParams (speed, serialPort.DATABITS_8, serialPort.STOPBITS_1, serialPort.PARITY_NONE);
    serialOutput = serialPort.getOutputStream ();
    serialInput = serialPort.getInputStream ();
    serialPort.notifyOnDataAvailable (true);
    }
    catch(Exception e)
    {
     System.out.println ("Creating Main:\n  " + e);
    }
  }

  public static void main (String argv[]) throws IOException
  {
    String comport = null;
    String uploadname = null;
    int comspeed = 38400;

    for (int i = 0; i < argv.length; i++)
      {
	if (argv[i].equalsIgnoreCase ("-port") && i + 1 < argv.length)
	  comport = argv[++i];
	else if (argv[i].equalsIgnoreCase ("-upload") && i + 1 < argv.length)
	  uploadname = argv[++i];
	else if (argv[i].equalsIgnoreCase ("-help"))
	  usage ();
	else if (argv[i].equalsIgnoreCase ("-speed") && i + 1 < argv.length)
	  {
	    try
	    {
	      comspeed = Integer.parseInt (argv[++i]);
	    }
	    catch (NumberFormatException nfe)
	    {
	      System.out.println ("Setting Serial Port Speed:\n  " + nfe);
	    }
	    System.out.println ("Communication speed set to " + comspeed + " bps.");
	  }
      }
    Main terminal = new Main ((comport != null) ? comport : "COM1", comspeed);
    terminal.openTerminalWindow ();
    terminal.instruments = new InstrumentPanel ();
    if (uploadname != null)
      new upLoad (uploadname, terminal.serialOutput);
  }
}
