/*
 * @(#)Shell.java   1.04 97/5/2  Martin Turon
 *
 * $Id$
 *
 *     Copyright (c) 1995-1997 Turon Technologies, Inc. 
 *                             All Rights Reserved.
 *
 * Permission to use, copy, and distribute this software and its 
 * documentation for NON-COMMERCIAL purposes and without fee is 
 * hereby granted provided that this copyright notice and the 
 * following license aggreement appears in all copies.
 *
 * This license is based on trust, that is, it is not enclosed in an 
 * encrypted envelope that forces the user to comply by its rules.
 * In understanding the design of the license pay in mind that certain 
 * language constructs are being used that would be very easy to 
 * implement in software.  A good example of this is the aspect of 
 * the license that involves revision history.  The license requires 
 * that whenever any user modifies the software, they will annotate 
 * the date, their name, a description of their improvements, and a 
 * return address to the bottom of the revision history list in the 
 * header of the source file (not too unlike a chain letter.)  The 
 * user is allowed to use and distribute the unmodified source and any 
 * unmodified compiled class files that result from its use.  If a user
 * wishes to distibute modified versions of the source, or executables 
 * made using modified source, user must pay ten ($10) dollars, equally 
 * divided to the final ten (10) authors in the revision history.  
 * Please note that this final rule of the license is what supports 
 * the authors of the document, and is only valid if the user is 
 * becoming a creator herself.  
 *
 * REVISION HISTORY:
 *    author            date      description        address
 *    ------            ----      -----------        -------
 *    martin@turon.com  7/26/96   Initial version.   PO Box 3051 
 *                                                   Danville, CA 
 *                                                   94526
 * z![C:/users/martin/code/java/zaBava/shell]> 
 *                          writing code is like casting spells!
 */

package zaBava.shell;

import java.awt.*;
import java.io.*;
import java.util.Date;
import java.util.Properties;
import zaBava.object.ObjArray;
import zaBava.shell.bin.Command;

/**
 * A command shell object.   
 *
 * @version 	1.04, 05/02/97
 * @author 	Martin Turon
 */
public class Shell extends TextArea {
   int    commandStart;      // Cursor position of command line.
   String drive;
   String directory;
   Properties props;

   // zapp>>>  @inherit ObjArray commandList of class ShellCommand 
   ObjArray	commandList = new ObjArray();

   public void add(ShellCommand child) {
	child.parent(this);
	commandList.add(child);
   }

   public void remove(ShellCommand child) {
	commandList.remove(child);
   }
   // <<<zapp  @inherit ObjArray of ShellCommand named commandList

  /***************** Shell ******/   
  /**
   * Treat the Shell like a TextArea...  Put it anywhere!
   */
   Shell() {
      //
      // add configuration file reader that sets
      // system properties and font and size info...
      //
      String startDir = "";
      try {
          props = System.getProperties();
          startDir = props.getProperty("user.dir");
      } catch (Exception e) {
      }
      setFont(new Font("Helvetica", Font.BOLD, 12));

      //
      // Initialize commandList with all the classes in
      // zaBava.bin.path
      //
      try {
      add(new ShellCommand(Class.forName("zaBava.shell.bin.exit"),
			   zaBava.shell.bin.exit.idPattern));
      add(new ShellCommand(Class.forName("zaBava.shell.bin.help"),
			   zaBava.shell.bin.help.idPattern));
      add(new ShellCommand(Class.forName("zaBava.shell.bin.ping"),
			   zaBava.shell.bin.ping.idPattern));
      add(new ShellCommand(Class.forName("zaBava.shell.bin.nslookup"),
			   zaBava.shell.bin.nslookup.idPattern));
      add(new ShellCommand(Class.forName("zaBava.shell.bin.hostname"),
			   zaBava.shell.bin.hostname.idPattern));
      add(new ShellCommand(Class.forName("zaBava.shell.bin.props"),
			   zaBava.shell.bin.props.idPattern));
      add(new ShellCommand(Class.forName("zaBava.shell.bin.ls"),
			   zaBava.shell.bin.ls.idPattern));
      add(new ShellCommand(Class.forName("zaBava.shell.bin.cat"),
			   zaBava.shell.bin.cat.idPattern));
      add(new ShellCommand(Class.forName("zaBava.shell.bin.cd"),
			   zaBava.shell.bin.cd.idPattern));
      } catch (Exception e) {}
      /*
      while(more bin commands left in user.path) {
         add(command);
      }
      */

      changeDirectory(startDir);
      // Initialize the text area with the login message:
      //     zaBava Session started:
      //       Wee Mon DD HH:MM:SS Year  
      String   d = new Date().toString();
      appendText("zaBava Session started:\n   "+d+"\n");

      // Don't use resetPrompt() in init, because since the 
      // Shell doesn't have the focus when it first comes up, 
      // getSelectionStart() returns 0 for the cursor position.
      resetPrompt();
      commandStart = 35+d.length();
      try {
          commandStart += drive.length()+directory.length();
      } catch (Exception e) {}
   }

   void resetCursor() {
      commandStart = getSelectionStart();
   }

   void resetPrompt() {
      appendText("\nz!["+drive+directory+"]> ");
      resetCursor();
   }

   String getCommandLine() {
      int cursor = getSelectionStart();
      if (cursor <= commandStart) return null;
      select(commandStart,cursor);
      return getSelectedText();      
   }
 
/*
 *
 * REVISION HISTORY:
 *	author	date	  description
 *	------	----	  -----------
 *	martin 	11/27/96  Initial version
 */
   public void changeDirectory(String path) {
      try {
          drive     = path.substring(0,2);
          directory = path.substring(2);
      } catch (Exception e) {}
      //    If you don't ask to *try* things like
      //    change some user's directory, the
      //    applet sandbox you're running in
      //    may attack the shell by throwing
      //    exceptions at it that may crash even
      //    innocent operations such as the vital
      //    displaying of output to the TextArea.
      try { 
	if (new File(path).isDirectory()) {
	   //
	   // As of JDK 1.1, simply setting the "user.dir" 
	   // System.property is *not* sufficient information
	   // to insure that future Runtime.exec() commands
	   // will start in the specified directory. :(
	   //
	   props.put("user.dir", path);
	   System.setProperties(props);

	} else {
	   System.out.println("   Invalid directory: "+path);
	}
      } catch (Exception e) {
	   System.out.println(e.toString());
      }
   }

/*
 *
 * REVISION HISTORY:
 *	author	date	  description
 *	------	----	  -----------
 *	martin 	12/10/96  Pulled into seperate routine, and added CAT
 *      martin  3/27/97   Added &, PROPS, EXIT, and environment passing to exec
 *      martin  4/01/97   Began conversion to generalized command execution
 *      martin  4/23/97   Converted to dynamic AI rule module system
 */
   void parseCommand(String command) {
     
     try {
	// pre parse command line and execute any known commands:
	//	CD, LS, CAT, EXIT, PROPS, MOVE, COPY, DEL 	
          if (command == null) {
            return;

	  // & -- Spawn Thread
	  //
	  } else if (command.startsWith("&")) {
	    Object Zverb    = new zaBava.shell.bin.spawn(this, 
						      command.substring(2));

	  // ? -- Help command
	  //
	  } else if (command.startsWith("?")) {
	    Object Zverb = new zaBava.shell.bin.help(this);

	  // X: -- Change drives
	  //
	  } else if (command.charAt(1) == ':') {
	    String letter = String.valueOf(command.charAt(0))+String.valueOf(command.charAt(1));
	    Object Zverb    = new zaBava.shell.bin.drive(this, letter);

	  // CD -- Change Directory
	  //
	  } else if (command.startsWith("cd")) {
	    String newDir   = command.substring(3);
	    Object Zverb    = new zaBava.shell.bin.cd(this, newDir);

	  // LS -- Hack to get directory listing to kinda work with CD.
	  //
	  } else if (command.startsWith("ls")) {
	    Object Zverb    = new zaBava.shell.bin.ls(this, command);

	  // CAT -- Dump the contents of a file
	  //
	  } else if (command.startsWith("cat")) {
	    String filename;
	    if ((drive != null) && (directory != null)) {
	       filename = drive+directory+"\\"+command.substring(4);
	    } else {
	       filename = command.substring(4);
	    }
	    Object Zverb    = new zaBava.shell.bin.cat(this, filename);

	  // EXIT -- quits out of this shell
	  //
	  } else if (command.startsWith("exit")) {
	    Object Zverb    = new zaBava.shell.bin.exit(this);

	  // PING -- checks if given machine is alive
	  //
	  } else if (command.startsWith("ping")) {
	    Object Zverb    = new zaBava.shell.bin.ping(this, 
							command.substring(5));

	  // PROPS -- Dump the system properties of this shell
	  //
	  } else if (command.startsWith("props")) {
	    Object Zverb    = new zaBava.shell.bin.props(this);
	  
	  // NSLOOKUP -- Perform reverse DNS lookup on given host
	  //
	  } else if (command.startsWith("nslookup")) {
	    Object Zverb    = new zaBava.shell.bin.nslookup(this,
							    command.substring(9));

	  // HOSTNAME -- Looks up the name of the local machine
	  //
	  } else if (command.startsWith("hostname")) {
	    Object Zverb    = new zaBava.shell.bin.hostname(this);

	  } else {
	    System.out.println("z!> "+command+"\n  AI scanning for patterns:");
	    // Replace this whole routine with next line of code!
	    // This is where the zaBava license gets interesting...
	    // Since I invented and was the first to use such an
	    // object messaging technique in java code, am I not
	    // entitled to a patent?  But what is a patent anyway
	    // in the modern day other than the right to sue people
	    // for a ton of money?  There must be an easier way
	    // to collect royalties on a good idea.  Forcing
	    // the software community at large to wait for good
	    // ideas due to twisted intellectual property laws 
	    // is detrimental.  We must figure out a way to reward 
	    // authors of good information directly!
            commandList.sendToChildren(new MSG_SHELL_COMMAND_PARSE(command));
	    //
 	    // if all else fails, System.exec() command in native runtime.
	    Object Zverb    = new zaBava.shell.bin.exec(this, command);

	  }

      } catch(Exception e) {
	 System.out.println(e);
      } 
   }

/*
 *
 * REVISION HISTORY:
 *	author	date	  description
 *	------	----	  -----------
 *	martin 	11/27/96  converted to command parsing
 */
   public void exec() {
	parseCommand(getCommandLine());
	resetPrompt();
   }


   public boolean keyDown(Event evt, int key) {
        boolean retVal = true;


	//if (Constant.DEBUG) {
	//   System.out.print("key: "+new Integer(key).toString()+"   ");
	//}
        switch (key) {
 	   case Constant.KEY_ESC:
	      // On ESC pressed, return to last command line. 
	      select(commandStart, commandStart);
	      break;

           case Constant.KEY_ENTER:
	      if (Constant.DEBUG) { System.out.println(); }
	      exec();
              break;

	   default:
	      // Quick way to find key constants:
              //   appendText(new Integer(key).toString());
	      // Send key presses we don't handle to superclass.
	  //  commandStart++; // fix for JDK1.1 bugs???
	      retVal = super.keyDown(evt,key);
	      break;
        }
        return retVal;
   }  


   public void appendStream(InputStream inStream) {
        int          nextChar;
        StringBuffer outString = new StringBuffer();

	try {
	    while ((nextChar = inStream.read()) != Constant.EOF)
	       outString.append((char)nextChar);
	    inStream.close();
	} catch(IOException e) {
	}
	appendText(outString.toString());
   }
}









